from fastapi import APIRouter, Depends, HTTPException, UploadFile, File, Form
from sqlmodel import Session
from backend.models.models import Shop, User
from backend.schemas.shop import ShopCreate, ShopRead
from backend.database import get_session
from backend.routes.auth import get_current_user
import shutil
import os

router = APIRouter()

static_dir = os.path.join("app", "static")
os.makedirs(static_dir, exist_ok=True)


@router.post("/", response_model=ShopRead)
def create_shop(
    name: str = Form(...),
    description: str = Form(None),
    file: UploadFile = File(None),
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
    #owner_id = int,

):
    shop = ShopCreate(name=name, description=description, owner_id=current_user.id)
    #shop = ShopCreate(name=name, description=description, owner_id=owner_id)
    db_shop = Shop.from_orm(shop)

    if file and file.filename:
        # Save the image to the static directory
        file_location = os.path.join(static_dir, f"{db_shop.name}_{file.filename}")
        with open(file_location, "wb") as buffer:
            shutil.copyfileobj(file.file, buffer)

        db_shop.image_url = file_location
    else:
        # Set a default image URL if no file is uploaded
        db_shop.image_url = os.path.join(static_dir, "default_shop_image.png")

    session.add(db_shop)
    session.commit()
    session.refresh(db_shop)

    # if file:
    #     # Delete the image file after session commit
    #     os.remove(file_location)
    return db_shop


@router.get("/list", response_model=list[ShopRead])
def get_all_shops(session: Session = Depends(get_session)):
    shops = session.query(Shop).all()
    return shops


@router.get("/{shop_id}", response_model=ShopRead)
def read_shop(shop_id: int, session: Session = Depends(get_session)):
    shop = session.get(Shop, shop_id)
    if not shop:
        raise HTTPException(status_code=404, detail="Shop not found")
    return shop


@router.put("/{shop_id}", response_model=ShopRead)
def update_shop(
    shop_id: int,
    name: str = Form(None),
    description: str = Form(None),
    file: UploadFile = File(None),
    session: Session = Depends(get_session),
):
    db_shop = session.get(Shop, shop_id)
    if not db_shop:
        raise HTTPException(status_code=404, detail="Shop not found")

    if name:
        db_shop.name = name
    if description:
        db_shop.description = description

    if file and file.filename:
        # Save the image to the static directory
        file_location = os.path.join(static_dir, f"{db_shop.name}_{file.filename}")
        with open(file_location, "wb") as buffer:
            shutil.copyfileobj(file.file, buffer)

        db_shop.image_url = file_location
    else:
        # Set a default image URL if no file is uploaded
        db_shop.image_url = os.path.join(static_dir, "default_shop_image.png")

    session.add(db_shop)
    session.commit()
    session.refresh(db_shop)

    if file:
        # Delete the image file after session commit
        os.remove(file_location)

    return db_shop


@router.delete("/{shop_id}")
def delete_shop(shop_id: int, session: Session = Depends(get_session)):
    shop = session.get(Shop, shop_id)
    if not shop:
        raise HTTPException(status_code=404, detail="Shop not found")
    session.delete(shop)
    session.commit()
    return {"message": "Shop deleted successfully"}
